#!/bin/sh
#----------------------------------------------------------------------------
# dhcp2ens - create ens.conf file                                      v2.0.8
#
# Creation:    14.04.2002  jed
# Last Update: 25.02.2004  jed
#
# Copyright (c) 2000-2001 Frank Meyer
#----------------------------------------------------------------------------

progname=dhcp2ens

rev_ip () {
    tmpipaddr="`echo $1 | sed 's/\./ /g'`"
    set -- $tmpipaddr
    rev_ip="$4.$3.$2.$1"
}

check_leases_file () {
    datestr=`dirname $ens_config_file`/date_`ls -l $leases_file|tr -s ' ' '_'|cut -d_ -f5-8`
    if [ ! -f $datestr ]; then
        ### file not exists ###
        rm -f `dirname $ens_config_file`/date_*
        echo "created by $progname" > $datestr
        return 1
    else
        ### file exists ###
        return 0
    fi
}

get_current_date_time () {
    ### get current date and time ###
    nowdate=`date -u +"%Y %m %d %H %M"`
    set -- `echo $nowdate`
    nowyear=$1
    nowmonth=$2
    nowday=$3
    nowhour=$4
    nowminute=$5
}

### extract information from leases file ###
create_leases_temp_file () {
    # remove temp file
    rm -f $leases_temp_file

    # remove comment, starts, hardware ethernet, uid lines
    # replace TAB, LF, CR characters by SPACES
    # replace '}' by a LF
    # remove 'abandoned' entries                                    NEU 01.04.2003 AM (Adi Markl)
    # remove '{', ';', '"' characters
    # remove lease, ends, client-hostname, hostname strings         AEN 01.04.2003 AM (Adi Markl)
    # remove EMPTY lines (also lines only containing WHITE_SPACE)   NEU 01.04.2003 AM (Adi Markl)
    # remove hardware token-ring!!                                  NEU 24.02.2003 PS (Peter Schauder)

    cat $leases_file                    \
    | grep -v "^[   ]*#"                \
    | grep -v "starts"                  \
    | grep -v "hardware ethernet"       \
    | grep -v "hardware token-ring"     \
    | grep -v "uid"                     \
    | tr '\011\012\015' '   '           \
    | tr '}' '\012'                     \
    | grep -v " abandoned;"             \
    | sed 's/{//g; s/;//g; s/"//g; s/   */ /g;
           s/^ *lease / /g;
           s/ ends [0-9] / /g;
           s/ client-hostname / /g;
           s/ hostname / /g'            \
    | grep -v "^[   ]*$"                \
    > $leases_temp_file

    # cp -p $leases_temp_file $leases_temp_file.1   #DBG
}

backup_ens_conf () {
    if [ -f $ens_config_file ]; then
        ### backup existing configuration file ###
        cp -fp $ens_config_file $ens_config_file.backup
    fi
}

print_entry () {
    ### check if entry (ip address) already exists ###
    grep "^[    ]*$ipaddr[      ]\+" $ens_config_file.new > /dev/null
    if [ $? -ne 0 ]; then           # if not ...
        ### check if host entry already exists ###
        grep -i "[      ]$host\.*" $ens_config_file.new > /dev/null
        if [ $? -ne 0 ]; then   # if not ...
            #### add dynamic dns entry ###
            rev_ip $ipaddr
            echo "$ipaddr                       $host.$DOMAIN_NAME"
            echo "PTR $rev_ip.in-addr.arpa      $host.$DOMAIN_NAME"
        fi
    fi
}

write_dynamic_ens_entries () {
    ### writing dynamic dns entries ###

    echo ""
    echo "### dynamic (dhcp) dns entries BEGIN ###"

    while read INPUTSTR; do
        ### Leere Zeile? berspringen!
        [ -z "$INPUTSTR" ] && continue

        set -- $INPUTSTR

        ### get ip address, end date, end time, hostname ###
        ipaddr=$1
        date=$2
        time=$3
        host=$4

        ### get lease end date ###
        set -- `echo $date|tr '/' ' '`
        leaseyear=$1
        leasemonth=$2
        leaseday=$3

        ### get lease end time ###
        set -- `echo $time|tr ':' ' '`
        leasehour=$1
        leaseminute=$2

        if [ "$leaseyear" -eq "$nowyear" ]; then
            if [ "$leasemonth" -eq "$nowmonth" ]; then
                if [ "$leaseday" -eq "$nowday" ]; then
                    if [ "$leasehour" -eq "$nowhour" ]; then
                        if [ "$leaseminute" -ge "$nowminute" ]; then
                            ### minute >= now => ok => finish
                            print_entry
                        fi

                    elif [ "$leasehour" -gt "$nowhour" ]; then
                        ### hour > now => new hour => finish
                        print_entry
                    fi

                elif [ "$leaseday" -gt "$nowday" ]; then
                    ### day > now => new day => finish
                    print_entry
                fi

            elif [ "$leasemonth" -gt "$nowmonth" ]; then
                ### month > now => new month => finish
                print_entry
            fi

        elif [ "$leaseyear" -gt "$nowyear" ]; then
            ### year > now => new year => finish
            print_entry
        fi
    done < $leases_temp_file

    echo "### dynamic (dhcp) dns entries END   ###"
    echo ""
}

write_ens_conf_new () {
    echo "creating ENS configuration file ..."

    (
    date=`date -u +"%d.%m.%Y %H:%M:%S"`
    echo "### $date created by $0 ###"
    echo ""

    echo "logfile   /ens.log"
    echo ""

    serial="`date +%Y%m%d`"                 # reverse date
    echo "SOA   $DOMAIN_NAME    $HOSTNAME.$DOMAIN_NAME  fli4l.$DOMAIN_NAME  $serial 60 60 60 0"
    echo "NS    $DOMAIN_NAME    $HOSTNAME.$DOMAIN_NAME"
    echo "ZONE  $DOMAIN_NAME"
    echo ""

    echo "127.0.0.1         localhost.$DOMAIN_NAME"
    echo "PTR 1.0.0.127.in-addr.arpa    localhost"

    i=1
    while [ "$i" -le "$HOSTS_N" ]; do
        set -- `eval echo '$HOST_'$i`
        ipaddr=$1
        host=$2

        aliases=''
        m=3
        max=$#
        while [ $m -le $max ]; do
            eval col='$'$m
            case "$col" in
                *:*)    # skip mac address
                    ;;
                *)      aliases="$aliases   $col"
                    ;;
            esac
            m=`expr $m + 1`
        done

        rev_ip $ipaddr
        echo "$ipaddr           $host.$DOMAIN_NAME  $aliases"
        echo "PTR $rev_ip.in-addr.arpa  $host.$DOMAIN_NAME"

        i=`/usr/bin/expr $i + 1`
    done

    write_dynamic_ens_entries

    i=1
    while [ $i -le $DNS_N ]; do
        dns="`eval echo '$DNS_'$i`"
        set -- $dns         # $1=firma.de,  $2=192.168.1.12
        echo "additional_forwarder  $2  $1"
        i=`/usr/bin/expr $i + 1`
    done

    for j in $DNS_FORWARDERS; do
        if [ "$j" != "0.0.0.0" ]; then
            echo "forwarder $j"
        fi
    done

    echo 'forward_entry_timeout 50'
    echo 'forward_next_timeout   5'

    i=1
    while [ $i -le $DNS_FORBIDDEN_N ]; do
        forbidden="`eval echo '$DNS_FORBIDDEN_'$i`"
        echo "forbidden $forbidden"
        i=`/usr/bin/expr $i + 1`
    done

    i=1
    while [ $i -le $IP_ETH_N ]; do
        net=`eval echo '$IP_ETH_'$i'_NETWORK' '$IP_ETH_'$i'_NETMASK'`
        dnet=`netcalc dnsnet $net`
        rnet=`netcalc dnsrev $net`

        echo "ACL   dns.allow   $dnet."
        echo "ZONE  $rnet"
        i=`/usr/bin/expr $i + 1`
    done

    for netm in $MASQ_NETWORK $ROUTE_NETWORK; do
        dnetm=`netcalc dnsnet $netm`

        found=false
        i=1
        while [ $i -le $IP_ETH_N ]; do
            net=`eval echo '$IP_ETH_'$i'_NETWORK' '$IP_ETH_'$i'_NETMASK'`
            dnet=`netcalc dnsnet $net`

            if [ $dnetm = $dnet ]; then
                found=true
                break
            fi

            i=`/usr/bin/expr $i + 1`
        done

        if [ $found = "false" ]; then 
            echo "ACL   dns.allow   $dnetm."
            rnetm=`netcalc dnsrev $netm`
            echo "ZONE  $rnetm"
        fi
    done

    if [ "$OPT_PPP" = "yes" ]; then
        echo "ACL   dns.allow   $PPP_PEER"
    fi

    if [ "$OPT_MGETTY" = "yes" ]; then
        echo "ACL   dns.allow   $MGETTY_PEER"
    fi

    echo "ACL   dns.allow   127.0.0.1"

    echo 'ACL   dns.deny    $'
    echo 'cache_max 10000'
    ) > $ens_config_file.new
}

activate_ens_conf () {
    if [ -f $ens_config_file.prev ]; then
        ### router currently online, process .prev file ###
        cp $ens_config_file.new $ens_config_file.prev

        grep -v "^[     ]*forwarder[    ]" $ens_config_file.new    > $ens_config_file
        grep    "^[     ]*forwarder[    ]" $ens_config_file.backup >> $ens_config_file
    else
        ###  router offline ###
        cp $ens_config_file.new $ens_config_file
    fi

    ### remove temp file ###
    rm -f $ens_config_file.new
}

force_ens_conf_reload () {
    # pid=`ps -ef|grep ens|grep -v grep|tr -s ' ' ' '|cut -d' ' -f1`
    # if [ "$pid" != "" ]; then
    #     ### force ens to reload config file ###
    #     kill -HUP $pid >/dev/null 2>&1
    # fi

    killall -HUP ens >/dev/null 2>&1
}

#----------------------------------------------------------------------------
# main
#----------------------------------------------------------------------------

rc_config_file=/etc/rc.cfg
. $rc_config_file

if [ "$DHCP2ENS_DO_DEBUG" = yes ]; then
    set -x
fi

### set variables ###
ens_config_file=/usr/local/ens/ens.conf
leases_temp_file=/usr/local/ens/dhcpd.tmp
leases_file="$DHCP2ENS_LEASES_FILE"

if [ "$OPT_DHCP2ENS" = "yes" ]; then
    if [ "$OPT_CRON" = "yes" -o "$OPT_EASYCRON" = "yes" ]; then
        ### cron daemon has been installed ###
        if [ -f $leases_file ]; then
            ### leases file exists ###
            check_leases_file
            if [ $? -eq 1 ]; then
                ### leases file has been modified => go on ###
                /usr/bin/logger -t "$progname" -p "local0.info" "updating ens.conf ..."
                get_current_date_time
                create_leases_temp_file
                backup_ens_conf 
                write_ens_conf_new
                activate_ens_conf

                ### set ownership, restart ens ###
                chown ens:ens $ens_config_file*     
                force_ens_conf_reload
            fi
        else
            ### lease file does not exist ###
            /usr/local/bin/colecho "$progname: no leases file found, exiting!" rd
            /usr/bin/logger -t "$progname" -p "local0.warning" "no leases file found, exiting!"
        fi
    else
        ### no cron daemon installed ###
        /usr/local/bin/colecho "$progname: no cron daemon installed, exiting!" rd
        /usr/bin/logger -t "$progname" -p "local0.warning" "no cron daemon installed, exiting!"
    fi
fi
rm -f $leases_temp_file

if [ "$DHCP2ENS_DO_DEBUG" = yes ]; then
    set +x
fi
#----------------------------------------------------------------------------
# End
#----------------------------------------------------------------------------


